﻿//version: 1.0

using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Windows.Forms;
using System.Reflection;
using System.Drawing;
using System.IO;
using System.Data.Odbc;

namespace ODBC
{

    public class ODBCConnection : IDisposable
    {
        private OdbcConnection ODBCCon;
        private OdbcTransaction ODBCTrans = null;
        public ODBCConnection()
        {
            ODBCCon = new OdbcConnection();

        }
        public void Open(string ConnectionString)
        {
            ODBCCon.ConnectionString = ConnectionString;
            ODBCCon.Open();

        }
        public void Close()
        {
            ODBCCon.Close();
        }
        public object Value
        {
            get
            {
                return ODBCCon;
            }
            set
            {
                ODBCCon = (OdbcConnection)value;
            }
        }
        public void BeginTransaction(object ODBCCommand)
        {
            ODBCTrans = ODBCCon.BeginTransaction();
            OdbcCommand ODBCCommand2 = (OdbcCommand)ODBCCommand;
            ODBCCommand2.Transaction = ODBCTrans;
        }
        public void EndTransaction()
        {
            ODBCTrans.Commit();
            ODBCTrans = null;
        }
        public void Dispose()
        {

            ODBCCon.Close();

        }

    }
    public class ODBCCommand : IDisposable
    {
        private OdbcCommand ODBCCmd;
        private double scaleX = 1;
        public ODBCCommand(string CommandText, OdbcConnection Connection)
        {
            ODBCCmd = new OdbcCommand(CommandText, Connection);
        }

        public string CommandText
        {
            get { return ODBCCmd.CommandText; }
            set
            {

                ODBCCmd.CommandText = value;
            }
        }


        public int ExecuteNonQuery()
        {
            ODBCCmd.Prepare();
            return ODBCCmd.ExecuteNonQuery();
        }
        public OdbcDataReader ExecuteReader()
        {
            ODBCCmd.Prepare();
            return ODBCCmd.ExecuteReader();
        }

        public string BytesToBLOB(Byte[] Data)
        {
            StringBuilder sb = new StringBuilder(Data.Length * 2 + 6);
            sb.Append("x\'");
            for (int i = 0; i < Data.Length; i++)
            {
                sb.Append(((int)Data[i]).ToString("x2"));
            }
            sb.Append("'");
            return sb.ToString();
        }
        public string FileToBLOB(string File)
        {
            byte[] buffer = null;
            using (FileStream fs = new FileStream(File, FileMode.Open))
            {
                buffer = new byte[fs.Length];
                fs.Read(buffer, 0, (int)fs.Length);
            }
            return BytesToBLOB(buffer);
        }
        public void ClearParameters()
        {
            ODBCCmd.Parameters.Clear();
        }
        public void AddParameter(string Name, OdbcType ODBCType)
        {
            ODBCCmd.Parameters.Add("@" + Name, ODBCType);
        }
        public void AddParameter(string Name, object ODBCType, int size)
        {
            ODBCCmd.Parameters.Add("@" + Name, (OdbcType)ODBCType, size);
        }
        public void SetParameter(string Name, string Value)
        {
            ODBCCmd.Parameters["@" + Name].Value = Value;
        }
        //public void SetNullParameter(string Name)
        //{
        //    command.Parameters["@" + Name].Value = null;
        //}
        public object Value
        {
            get { return ODBCCmd; }
            set { ODBCCmd = (OdbcCommand)value; }
        }
        public void ExecuteTable(DataGrid Table, int Maximum)
        {
            Type t2 = Table.GetType();
            FieldInfo fi = t2.GetField("dataTable");
            DataTable dt = (DataTable)fi.GetValue(Table);
            Table.DataSource = null;
            dt.DefaultView.RowFilter = "";
            dt.DefaultView.Sort = "";
            dt.Columns.Clear();
            dt.Rows.Clear();
            Table.TableStyles[0].GridColumnStyles.Clear();
            using (OdbcDataReader r = ODBCCmd.ExecuteReader())
            {
                object[] nullReplace = new object[r.FieldCount];
                for (int i = 0; i < r.FieldCount; i++)
                {
                    string colName = r.GetName(i);
                    DataGridColumnStyle dg = NewColumnStyle(colName);
                    dg.Width = (int)(75 * scaleX);
                    Table.TableStyles[0].GridColumnStyles.Add(dg);
                    Type t = r.GetFieldType(i);
                    if (t == typeof(string) || t == typeof(char) || t == typeof(bool) || t.IsArray || t == typeof(DateTime)
                    || t == typeof(Guid))
                    {
                        dt.Columns.Add(colName, typeof(string));
                        nullReplace[i] = "";
                    }
                    else
                    {
                        dt.Columns.Add(colName, typeof(double));
                        nullReplace[i] = 0.0;
                    }
                }
                int i2 = 1;
                object[] o = new object[r.FieldCount];
                while (r.Read() && i2++ != Maximum)
                {
                    r.GetValues(o);
                    for (int i = 0; i < o.Length; i++)
                    {
                        if (DBNull.Value.Equals(o[i]))
                            o[i] = nullReplace[i];
                    }
                    DataRow dr = dt.NewRow();
                    dr.ItemArray = o;
                    dt.Rows.Add(dr);
                }
                dt.EndLoadData();
                Table.DataSource = dt.DefaultView;
                r.Close();
            }

        }

        private DataGridTextBoxColumn NewColumnStyle(string name)
        {
            DataGridTextBoxColumn dg = new DataGridTextBoxColumn();
            dg.MappingName = name;
            dg.HeaderText = name;
            return dg;
        }

        void IDisposable.Dispose()
        {
            ODBCCmd.Dispose();
        }
        public const OdbcType typeBigInt = OdbcType.BigInt;
        public OdbcType typeBinary = OdbcType.Binary;
        public OdbcType typeBit = OdbcType.Bit;
        public OdbcType typeChar = OdbcType.Char;
        public OdbcType typeDate = OdbcType.Date;
        public OdbcType typeDateTime = OdbcType.DateTime;
        public OdbcType typeDecimal = OdbcType.Decimal;
        public const OdbcType typeDouble = OdbcType.Double;
        public OdbcType typeImage = OdbcType.Image;
        public OdbcType typeInt = OdbcType.Int;
        public OdbcType typeNChar = OdbcType.NChar;
        public OdbcType typeNText = OdbcType.NText;
        public OdbcType typeNumeric = OdbcType.Numeric;
        public OdbcType typeNVarChar = OdbcType.NVarChar;
        public OdbcType typeReal = OdbcType.Real;
        public OdbcType typeSmallDateTime = OdbcType.SmallDateTime;
        public OdbcType typeSmallInt = OdbcType.SmallInt;
        public OdbcType typeText = OdbcType.Text;
        public OdbcType typeTime = OdbcType.Time;
        public OdbcType typeTimestamp = OdbcType.Timestamp;
        public OdbcType typeTinyInt = OdbcType.TinyInt;
        public OdbcType typeUniqueIdentifier = OdbcType.UniqueIdentifier;
        public OdbcType typeVarBinary = OdbcType.VarBinary;
        public OdbcType typeVarChar = OdbcType.VarChar;
    }
    public class ODBCReader : IDisposable
    {
        private OdbcDataReader ODBCRead;
        public ODBCReader()
        {

        }
        public object Value
        {
            get
            {
                return ODBCRead;
            }
            set
            {
                ODBCRead = (OdbcDataReader)value;
            }
        }
        public bool ReadNextRow()
        {
            return ODBCRead.Read();
        }
        public string GetValue(int Index)
        {
            return ODBCRead.GetValue(Index).ToString();
        }
        public byte[] GetBytes(int Index)
        {
            long size = ODBCRead.GetBytes(Index, 0, null, 0, 0);
            byte[] buffer = new byte[size];
            ODBCRead.GetBytes(Index, 0, buffer, 0, (int)size);
            return buffer;
        }
        public Bitmap GetImage(int Index)
        {
            Bitmap b = null;
            using (MemoryStream ms = new MemoryStream())
            {
                byte[] buffer = GetBytes(Index);
                ms.Write(buffer, 0, buffer.Length);
                b = new Bitmap(ms);
            }
            return b;
        }
        public int FieldCount
        {
            get { return ODBCRead.FieldCount; }
        }
        public String[] FieldNames()
        {

            String[] ODBCFields = new String[ODBCRead.FieldCount];
            int i;
            for (i = 0; i < ODBCRead.FieldCount; i++)
            {
                ODBCFields[i] = ODBCRead.GetName(i);
            }
            return ODBCFields;

        }
        public string FieldType(int Pos)
        {
            if (Pos > ODBCRead.FieldCount - 1)
                return "";
            else
                return ODBCRead.GetFieldType(Pos).ToString();

        }
        public bool IsDBNull(int index)
        {
            return ODBCRead.IsDBNull(index);
        }
        public void Close()
        {
            ODBCRead.Close();
        }
        public void Dispose()
        {
            if (ODBCRead != null)
                ODBCRead.Close();
        }
    }
}
